/*===========================================================================
 *
 * File:	Sample.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	May 10, 2003
 *
 * Sample project demonstrating use of the Morrowind CEsmFile and related
 * classes. Read the sample functions below for a general idea of how to use
 * the file and record classes. For details on each specific record, read
 * the record/sub-record source/include files. 
 *
 *=========================================================================*/

	/* Include Files */
#include "esmfile.h"


DEFINE_FILE("Sample.cpp");

/*===========================================================================
 *
 * Function - void LoadSaveEsp (void);
 *
 * Sample which simply loads an existing ESP/ESM file and saves it to 
 * another file.
 *
 *=========================================================================*/
void LoadSaveEsp (void) {
  CEsmFile TestFile;
  CEsmFile NewFile;
  bool     Result;

	/* Read test file, abort on any error */
  Result = TestFile.Read("test.esp");
  if (!Result) return;

	/* Write file, abort on any error */
  Result = TestFile.Write("backup.esp");
  if (!Result) return;

	/* Create a new file from scratch. All existing records in the NewFile
	 * are removed and the file is initialized with the required information
	 * (such as the TES3 header record, etc...). The MAST records are not
	 * added automatically by the CEsmFile class. */
  NewFile.CreateNew();

 }
/*===========================================================================
 *		End of Function LoadSaveEsp()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - void EditRecords (void);
 *
 * Sample function demonstrating how to add, edit, and remove records from
 * the ESP/ESM file.
 *
 *=========================================================================*/
void EditRecords (void) {
  CEsmFile       TestFile;
  CEsmRecord*    pRecord;
  CEsmContainer* pContainer;
  bool           Result;

	/* Read test file, abort on any error */
  Result = TestFile.Read("test.esp");
  if (!Result) return;

	/* Edit record */
  if (TestFile.GetNumRecords() > 0) {
    pRecord = TestFile.GetRecord(0);
    pRecord->SetBlocked(true);
    pRecord->SetPersist(true);
   }

	/* Add and initialize a new Container record */
  pRecord = TestFile.AllocateRecord(MWESM_REC_CONT);
  pRecord->SetID("my_test_cont_01");
  pContainer = (CEsmContainer *) pRecord;
  pContainer->SetName("Test Container");
  pContainer->SetWeight(500);

	/* Create a copy of an existing record. The new record is not added to the
	 * TestFile. It must be done manually if desired. */
  pRecord = TestFile.CreateCopy(pContainer);
  TestFile.AddRecord(pRecord);

	/* Delete the new container records */
  TestFile.DeleteRecord(pContainer);
  TestFile.DeleteRecord(pRecord);
 }
/*===========================================================================
 *		End of Function EditRecords()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - void FindRecords (void);
 *
 * Sample function demonstrating how to find or enumerate through the 
 * records in a ESP/ESM file.
 *
 *=========================================================================*/
void FindRecords (void) {
  CEsmFile      TestFile;
  CEsmRecord*   pRecord;
  bool          Result; 
  int		ArrayIndex;

	/* Read test file, abort on any error */
  Result = TestFile.Read("test.esp");
  if (!Result) return;

	/* Start finding all misc. items. Record name definitions can be found
	 * in the EsmDefs.h include file. */
  pRecord = TestFile.FindFirst(MWESM_REC_MISC, ArrayIndex);

  while (pRecord != NULL) {
    printf ("Misc ID = %s", pRecord->GetID());
    pRecord = TestFile.FindNext(MWESM_REC_MISC, ArrayIndex);
   }

 }
/*===========================================================================
 *		End of Function FindRecords()
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Main Program
 *
 *=========================================================================*/
int main (void) {
  return (0);
 }
/*===========================================================================
 *		End of Main Program
 *=========================================================================*/

